<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\HookHandlers;

use CleverReach\WooCommerce\Components\Repositories\Double_Opt_In_Repository;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\Automation\Cart_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\Automation\Site_Automation_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\Configuration\Config_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\Customers\Subscriber_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\DOI\Double_Opt_In_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\Tag\Tag_Service;
use CleverReach\WooCommerce\Components\Util\Shop_Helper;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\DoubleOptIn\DTO\DoubleOptInEmail;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\DoubleOptIn\Tasks\SendDoubleOptInEmailsTask;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Group\Contracts\GroupService;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Contracts\RecoveryEmailStatus;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Entities\AutomationRecord;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Entities\CartAutomation;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Exceptions\FailedToCreateAutomationRecordException;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Exceptions\FailedToDeleteAutomationRecordException;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Exceptions\FailedToUpdateAutomationRecordException;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Interfaces\AutomationRecordService;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Receiver\DTO\Receiver;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Receiver\Tasks\Composite\Configuration\SyncConfiguration;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Receiver\Tasks\Composite\ReceiverSyncTask;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Receiver\Tasks\Composite\SubscribeReceiverTask;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Receiver\Tasks\Composite\UnsubscribeReceiverTask;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\TaskExecution\QueueService;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Configuration\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Serializer\Serializer;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\QueueItem;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\QueueService as Base_Queue_Service;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Task;
use DateTime;
use Exception;

/**
 * Class Base_Handler
 *
 * @package CleverReach\WooCommerce\Components\HookHandlers
 */
abstract class Base_Handler {

	/**
	 * Tag Service
	 *
	 * @var Tag_Service
	 */
	private $tag_service;

	/**
	 * Config Service
	 *
	 * @var Config_Service
	 */
	private $config_service;

	/**
	 * Subscriber Service
	 *
	 * @var Subscriber_Service
	 */
	private $receiver_service;

	/**
	 * Double Opt In service
	 *
	 * @var Double_Opt_In_Service
	 */
	private $doi_service;

	/**
	 * Automation record service
	 *
	 * @var AutomationRecordService
	 */
	private $automation_record_service;

	/**
	 * Cart service
	 *
	 * @var Cart_Service
	 */
	private $cart_service;

	/**
	 * Check whether event should be handled or not.
	 *
	 * @return bool
	 */
	protected function should_handle_event() {
		return Shop_Helper::is_plugin_enabled()
			   && $this->is_initial_sync_task_completed()
			   && Shop_Helper::is_woocommerce_active();
	}

	/**
	 * Enqueue task
	 *
	 * @param Task $task Task.
	 */
	protected function enqueue_task( Task $task ) {
		if ( ! $this->is_initial_sync_task_completed() ) {
			return;
		}

		try {
			/**
			 *  Queue service
			 *
			 * @var QueueService $queue_service
			 */
			$queue_service = ServiceRegister::getService( Base_Queue_Service::CLASS_NAME );
			$queue_service->enqueue( $this->get_config_service()->getDefaultQueueName(), $task );
		} catch ( QueueStorageUnavailableException $ex ) {
			Logger::logDebug(
				json_encode(
					array(
						'Message'          => 'Failed to enqueue task ' . $task->getType(),
						'ExceptionMessage' => $ex->getMessage(),
						'ExceptionTrace'   => $ex->getTraceAsString(),
						'ShopData'         => Serializer::serialize( $task ),
					)
				),
				'Integration'
			);
		}
	}

	/**
	 * Sends confirmation email
	 *
	 * @param string $email Email.
	 * @param array  $tags_for_delete Tags for delete.
	 */
	protected function send_confirmation_email( $email, $tags_for_delete = array() ) {
		$doi_repository = new Double_Opt_In_Repository();

		if ( ! $doi_repository->does_doi_task_exist_for_email( $email ) ) {
			$form_id = $this->get_config_service()->get_default_form();

			$doi_email = new DoubleOptInEmail(
				$form_id,
				'activate',
				$email,
				$this->get_doi_service()->create_doi_data()
			);

			$this->enqueue_task( new ReceiverSyncTask( new SyncConfiguration( array( $email ), $tags_for_delete ) ) );
			$this->enqueue_task( new SendDoubleOptInEmailsTask( array( $doi_email ) ) );
		}
	}

	/**
	 * Sets wp_cr_newsletter_status to 1
	 *
	 * @param Receiver $subscriber Subscriber.
	 * @param array    $tags_for_delete Tags for delete.
	 */
	protected function activate_subscriber( $subscriber, $tags_for_delete = array() ) {
		try {
			$subscriber->setActive( true );
			$this->get_subscriber_service()->update_subscriber( $subscriber );

			$this->enqueue_task(
				new ReceiverSyncTask(
					new SyncConfiguration(
						array( $subscriber->getEmail() ),
						$tags_for_delete
					)
				)
			);
			$this->enqueue_task( new SubscribeReceiverTask( $subscriber->getEmail() ) );
		} catch ( Exception $e ) {
			Logger::logError(
				'Failed to get subscriber by email or update subscriber.',
				'Integration',
				array(
					'message' => $e->getMessage(),
					'trace'   => $e->getTraceAsString(),
				)
			);
		}
	}

	/**
	 * Sets wp_cr_newsletter_status to 0
	 *
	 * @param Receiver $subscriber Subscriber.
	 * @param array    $tags_for_delete Tags for delete.
	 */
	protected function deactivate_subscriber( $subscriber, $tags_for_delete = array() ) {
		try {
			$subscriber->setActive( false );
			$this->get_subscriber_service()->update_subscriber( $subscriber );

			$this->enqueue_task(
				new ReceiverSyncTask(
					new SyncConfiguration(
						array( $subscriber->getEmail() ),
						$tags_for_delete
					)
				)
			);
			$this->enqueue_task( new UnsubscribeReceiverTask( $subscriber->getEmail() ) );
		} catch ( Exception $e ) {
			Logger::logError(
				'Failed to get subscriber by email or update subscriber.',
				'Integration',
				array(
					'message' => $e->getMessage(),
					'trace'   => $e->getTraceAsString(),
				)
			);
		}
	}

	/**
	 * Retrieves config service
	 *
	 * @return Config_Service
	 */
	protected function get_config_service() {
		if ( null === $this->config_service ) {
			$this->config_service = ServiceRegister::getService( Configuration::CLASS_NAME );
		}

		return $this->config_service;
	}

	/**
	 * Retrieves config service
	 *
	 * @return Subscriber_Service
	 */
	protected function get_subscriber_service() {
		if ( null === $this->receiver_service ) {
			$this->receiver_service = ServiceRegister::getService( Subscriber_Service::CLASS_NAME );
		}

		return $this->receiver_service;
	}

	/**
	 * Retrieves Double Opt In Service
	 *
	 * @return Double_Opt_In_Service
	 */
	protected function get_doi_service() {
		if ( null === $this->doi_service ) {
			$this->doi_service = new Double_Opt_In_Service();
		}

		return $this->doi_service;
	}

	/**
	 * Retrieves Tag Service
	 *
	 * @return Tag_Service
	 */
	protected function get_tag_service() {
		if ( ! $this->tag_service ) {
			$this->tag_service = new Tag_Service();
		}

		return $this->tag_service;
	}

	/**
	 * Delete automation record
	 *
	 * @param string $cart_id Cart id (session_key).
	 */
	protected function delete_automation_record( $cart_id ) {
		if ( empty( $cart_id ) ) {
			return;
		}

		try {
			$this->get_automation_record_service()
				->deleteBy(
					array(
						'cartId' => (string) $cart_id,
						'status' => RecoveryEmailStatus::PENDING,
					)
				);
		} catch ( FailedToDeleteAutomationRecordException $e ) {
			Logger::logError(
				'Failed to delete cart record.',
				'Integration',
				array(
					'message' => $e->getMessage(),
					'trace'   => $e->getTraceAsString(),
					'cartId'  => $cart_id,
				)
			);
		}
	}

	/**
	 * Create or update automation record
	 *
	 * @param array      $cart_items Array of cart items.
	 * @param float      $total Cart total.
	 * @param string     $email Customer email.
	 * @param string|int $cart_id Cart ID.
	 * @param bool       $refresh_schedule If AC schedule should be refreshed.
	 */
	protected function create_or_update_automation_record( $cart_items, $total, $email, $cart_id, $refresh_schedule = false ) {
		if ( ( empty( $email ) && ! $refresh_schedule ) || empty( $cart_items ) ) {
			return;
		}

		$automation_service = new Site_Automation_Service();
		$automation         = $automation_service->get();
		if ( null === $automation || ! $automation->isActive() || 'created' !== $automation->getStatus() ) {
			return;
		}

		$record = $this->get_automation_record_service()
					->findBy(
						array(
							'cartId' => (string) $cart_id,
							'status' => RecoveryEmailStatus::PENDING,
						)
					);

		if ( 0 < count( $record ) ) {
			try {
				$record = $record[0];
				if ( ! empty( $email ) && $email !== $record->getEmail() ) {
					$record->setEmail( $email );
				}

				if ( $refresh_schedule ) {
					$record->setScheduledTime( new DateTime() );
					$record->setAmount( $total );
					$this->automation_record_service->refreshScheduleTime( $record );
				}

				$this->automation_record_service->update( $record );
			} catch ( FailedToUpdateAutomationRecordException $e ) {
				Logger::logError(
					'Failed to refresh schedule time on automation record.',
					'Integration',
					array(
						'message' => $e->getMessage(),
						'trace'   => $e->getTraceAsString(),
					)
				);
			}
		} else {
			if ( empty( $email ) ) {
				return;
			}

			$this->create_record( $automation, $email, $cart_id, $total );
		}
	}

	/**
	 * Retrieves AutomationRecordService.
	 *
	 * @return AutomationRecordService|object
	 */
	protected function get_automation_record_service() {
		if ( null === $this->automation_record_service ) {
			$this->automation_record_service = ServiceRegister::getService( AutomationRecordService::CLASS_NAME );
		}

		return $this->automation_record_service;
	}

	/**
	 * Returns group service instance.
	 *
	 * @return GroupService
	 */
	protected function get_group_service() {
		/** @noinspection PhpIncompatibleReturnTypeInspection */
		return ServiceRegister::getService( GroupService::CLASS_NAME );
	}

	/**
	 * Return cart service instance.
	 *
	 * @return Cart_Service
	 */
	protected function get_cart_service() {
		if ( null === $this->cart_service ) {
			$this->cart_service = new Cart_Service();
		}

		return $this->cart_service;
	}

	/**
	 * Creates automation record.
	 *
	 * @param CartAutomation $automation Automation.
	 * @param string         $email Customer email.
	 * @param string|int     $cart_id Cart ID.
	 * @param float          $total Cart total.
	 */
	private function create_record( CartAutomation $automation, $email, $cart_id, $total ) {
		try {
			$record = new AutomationRecord();
			$record->setAutomationId( $automation->getId() );
			$record->setCartId( (string) $cart_id );
			$record->setGroupId( $this->get_group_service()->getId() );
			$record->setEmail( $email );
			$record->setAmount( $total );

			$this->automation_record_service->create( $record );
		} catch ( FailedToCreateAutomationRecordException $e ) {
			Logger::logError(
				'Failed to create cart record.',
				'Integration',
				array(
					'message' => $e->getMessage(),
					'trace'   => $e->getTraceAsString(),
				)
			);
		}
	}

	/**
	 * Retrieves whether initial sync task in enqueued.
	 *
	 * @return bool
	 */
	private function is_initial_sync_task_completed() {
		/**
		 * Queue service
		 *
		 * @var QueueService $queue_service
		 */
		$queue_service     = ServiceRegister::getService( Base_Queue_Service::CLASS_NAME );
		$initial_sync_task = $queue_service->findLatestByType( 'InitialSyncTask' );

		return null !== $initial_sync_task && $initial_sync_task->getStatus() === QueueItem::COMPLETED;
	}
}
