<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\Services\BusinessLogic\Automation;

use CleverReach\WooCommerce\Components\Services\BusinessLogic\Automation\Trigger\Trigger_Newsletter_Subscription_Task;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\Configuration\Config_Service;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\ORM\Interfaces\ConditionallyDeletes;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Scheduler\Models\MinuteSchedule;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Scheduler\Models\Schedule;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Configuration\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Configuration\ConfigurationManager;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ORM\Exceptions\QueryFilterInvalidParamException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ORM\Interfaces\RepositoryInterface;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ORM\QueryFilter\Operators;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ORM\QueryFilter\QueryFilter;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ORM\RepositoryRegistry;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\TimeProvider;
use Exception;

/**
 * Class Checkbox_Service
 *
 * @package CleverReach\WooCommerce\Components\Services\BusinessLogic\Automation
 */
class Newsletter_Subscription_Service {

	/**
	 * Config Service
	 *
	 * @var Config_Service
	 */
	private $config_service;

	/**
	 * Schedule task for user subscription.
	 *
	 * @param string $billing_email Billing email.
	 *
	 * @return int
	 */
	public function subscribe( $billing_email ) {
		try {
			$queue_name = $this->get_config_service()->getDefaultQueueName();
			$context    = $this->get_config_manager()->getContext();

			$schedule = new MinuteSchedule( new Trigger_Newsletter_Subscription_Task( $billing_email ), $queue_name, $context );

			$current_time = $this->get_time_provider()->getCurrentLocalTime();
			$target_time  = $current_time->modify( '+30 seconds' );
			$schedule->setHour( (int) $target_time->format( 'G' ) );
			$schedule->setMinute( (int) $target_time->format( 'i' ) );
			$schedule->setRecurring( false );
			$schedule->setNextSchedule();

			return $this->get_schedule_repository()->save( $schedule );
		} catch ( Exception $e ) {
			Logger::logError(
				'Failed to handle CleverReach newsletter subscription checked event.',
				'Integration',
				array(
					'message' => $e->getMessage(),
					'trace'   => $e->getTraceAsString(),
				)
			);

			return 0;
		}
	}

	/**
	 * Delete scheduled task for user subscription.
	 *
	 * @param string | int $task_id Scheduled task id.
	 */
	public function undo( $task_id ) {
		try {
			$query = new QueryFilter();
			$query->where( 'id', Operators::EQUALS, (int) $task_id );

			$schedule = $this->get_schedule_repository()->selectOne( $query );

			isset( $schedule ) && $this->get_schedule_repository()->delete( $schedule );
		} catch ( QueryFilterInvalidParamException $e ) {
			Logger::logError(
				'Failed to undo.',
				'Integration',
				array(
					'message' => $e->getMessage(),
					'trace'   => $e->getTraceAsString(),
				)
			);
		}
	}

	/**
	 * Retrieves config service
	 *
	 * @return Config_Service
	 */
	private function get_config_service() {
		if ( null === $this->config_service ) {
			$this->config_service = ServiceRegister::getService( Configuration::CLASS_NAME );
		}

		return $this->config_service;
	}

	/**
	 * Retrieves configuration manager.
	 *
	 * @return ConfigurationManager | object
	 */
	private function get_config_manager() {
		return ServiceRegister::getService( ConfigurationManager::CLASS_NAME );
	}

	/**
	 * Retrieves time provider.
	 *
	 * @return TimeProvider | object
	 */
	private function get_time_provider() {
		return ServiceRegister::getService( TimeProvider::CLASS_NAME );
	}

	/**
	 * Retrieves schedule repository.
	 *
	 * @return RepositoryInterface | ConditionallyDeletes
	 *
	 * @noinspection PhpDocMissingThrowsInspection
	 */
	private function get_schedule_repository() {
		/** @noinspection PhpUnhandledExceptionInspection */
		return RepositoryRegistry::getRepository( Schedule::getClassName() );
	}
}
