<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Controllers;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use CleverReach\WooCommerce\Components\Exceptions\Invalid_Sync_Settings_Exception;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\Order_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\SyncSettings\Buyer_Sync_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\SyncSettings\Contact_Sync_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\SyncSettings\Subscriber_Sync_Service;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\SyncSettings\Sync_Settings_Service;
use CleverReach\WooCommerce\Components\Util\HTTP_Helper;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Order\Contracts\OrderService as Base_Order_Service;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\SyncSettings\Contracts\SyncSettingsService as Base_Sync_Settings_Service;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;

/**
 * Class Clever_Reach_Sync_Settings_Controller
 *
 * @package CleverReach\WooCommerce\Controllers
 */
class Clever_Reach_Sync_Settings_Controller extends Clever_Reach_Base_Controller {

	/**
	 * Sync settings service.
	 *
	 * @var Sync_Settings_Service $sync_service
	 */
	private $sync_settings_service;

	/**
	 * Saves sync settings.
	 */
	public function save_sync_settings() {
		try {
			$this->save_services();
			$this->return_json( array( 'success' => true ) );
		} catch ( Invalid_Sync_Settings_Exception $e ) {
			$this->return_json(
				array(
					'success' => false,
					'message' => $e->getMessage(),
				)
			);
		}
	}

	/**
	 * Retrieves sync settings
	 */
	public function get_sync_settings() {
		$enabled_services = $this->get_enabled_services_formatted();
		$this->return_json( $enabled_services );
	}

	/**
	 * Saves services
	 *
	 * @throws Invalid_Sync_Settings_Exception Exception if invalid sync settings.
	 */
	private function save_services() {
		$settings      = HTTP_Helper::get_param( 'syncSettings' );
		$sync_settings = explode( ',', $settings );

		$this->validate( $sync_settings );

		$enabled_services = array(
			ServiceRegister::getService( Subscriber_Sync_Service::CLASS_NAME ),
		);

		if ( in_array( 'buyers', $sync_settings, true ) ) {
			$enabled_services[] = ServiceRegister::getService( Buyer_Sync_Service::CLASS_NAME );
		}

		if ( in_array( 'contacts', $sync_settings, true ) ) {
			$enabled_services[] = ServiceRegister::getService( Contact_Sync_Service::CLASS_NAME );
		}

		$this->get_sync_settings_service()->setEnabledServices( $enabled_services );
	}

	/**
	 * Validates sync settings params
	 *
	 * @param array $sync_settings Array of sync settings.
	 *
	 * @throws Invalid_Sync_Settings_Exception Exception if invalid sync settings.
	 */
	private function validate( $sync_settings ) {
		if ( ! in_array( 'subscribers', $sync_settings, true ) ) {
			throw new Invalid_Sync_Settings_Exception( __( 'Subscribers must be selected.', 'cleverreach-wc' ) );
		}

		if ( in_array( 'contacts', $sync_settings, true ) &&
			 ! in_array( 'buyers', $sync_settings, true ) ) {
			throw new Invalid_Sync_Settings_Exception(
				__(
					'Contacts cannot be selected without buyers.',
					'cleverreach-wc'
				)
			);
		}
	}

	/**
	 * Returns enabled services
	 *
	 * @return array
	 */
	private function get_enabled_services_formatted() {
		$buyer            = false;
		$contact          = false;
		$enabled_services = $this->get_sync_settings_service()->getEnabledServices();

		foreach ( $enabled_services as $service ) {
			if ( $service->getUuid() === 'buyer-service' ) {
				$buyer = true;
			}

			if ( $service->getUuid() === 'contact-service' ) {
				$contact = true;
			}
		}

		return array(
			'buyers'   => $buyer,
			'contacts' => $contact,
		);
	}

	/**
	 * Retrieves Sync settings service.
	 *
	 * @return Sync_Settings_Service
	 */
	private function get_sync_settings_service() {
		if ( null === $this->sync_settings_service ) {
			$this->sync_settings_service = ServiceRegister::getService( Base_Sync_Settings_Service::CLASS_NAME );
		}

		return $this->sync_settings_service;
	}

	/**
	 * Retrieves order service.
	 *
	 * @return Order_Service
	 */
	protected function get_order_service() {
		/** @noinspection PhpIncompatibleReturnTypeInspection */
		return ServiceRegister::getService( Base_Order_Service::CLASS_NAME );
	}
}
